'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.fetchError = fetchError;
exports.updateDataSource = updateDataSource;
exports.applyHeaders = applyHeaders;
exports.applyToken = applyToken;
exports.urlWithParams = urlWithParams;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

/*
 * Actions
 */

var DATASOURCE_REQUEST = 'DATASOURCE_REQUEST';
exports.DATASOURCE_REQUEST = DATASOURCE_REQUEST;
var DATASOURCE_RESPONSE = 'DATASOURCE_RESPONSE';
exports.DATASOURCE_RESPONSE = DATASOURCE_RESPONSE;
var DATASOURCE_ERROR = 'DATASOURCE_ERROR';
exports.DATASOURCE_ERROR = DATASOURCE_ERROR;
var FETCH_ERROR = 'FETCH_ERROR';

exports.FETCH_ERROR = FETCH_ERROR;
function dataSourceRequest(path, params) {
    return {
        type: DATASOURCE_REQUEST,
        path: path,
        params: params
    };
}

function dataSourceResponse(path, response, params) {
    return {
        type: DATASOURCE_RESPONSE,
        path: path,
        data: response.data,
        meta: response.meta,
        params: params
    };
}

function fetchError(response, id) {
    //logging, react-router pushState
    return {
        type: FETCH_ERROR,
        response: response,
        id: id
    };
}

/*
 * Action Creators
 */

function updateDataSource(path, newParams) {
    var endpoint = arguments.length <= 2 || arguments[2] === undefined ? '' : arguments[2];

    return function (dispatch, getState) {
        var state = getState();

        var token = state.auth.token;

        // We'll store the new filter/sort/page state in this variable until we can be sure the data source is successfully updated.
        var params = _extends({}, state.dataSources[path] ? state.dataSources[path].params : {}, newParams);

        var requestOptions = {
            method: 'GET',
            mode: 'cors',
            headers: {
                'Content-type': 'application/json',
                'Accept': 'application/json'
            }
        };

        dispatch(dataSourceRequest(path, params));

        return (0, _isomorphicFetch2['default'])(urlWithParams(path, params, endpoint), applyToken(requestOptions, token)).then(function (response) {
            return response.json();
        }).then(function (json) {
            dispatch(dataSourceResponse(path, json, params));
        })['catch'](function (response) {
            return dispatch(fetchError(response));
        });
    };
}

/**
 * Helper Functions
 */

function applyHeaders(request, headers) {
    return _extends({}, request, { headers: _extends({}, request.headers, headers) });
}

function applyToken(request, token) {
    return applyHeaders(request, { 'Authorization': 'Bearer ' + token });
}

function urlWithParams(path) {
    var params = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
    var endpoint = arguments.length <= 2 || arguments[2] === undefined ? '' : arguments[2];

    var formattedParams = [];

    for (var key in params) {
        if (params.hasOwnProperty(key)) {
            formattedParams.push(key + '=' + params[key]);
        }
    }

    // If we have some params to add to the URL, append everything with a ?, otherwise
    // just return the url
    if (formattedParams.length == 0) {
        return buildUrl(endpoint, path);
    } else {
        return buildUrl(endpoint, path) + '?' + formattedParams.join('&');
    }
}

function buildUrl(base, path) {
    if (base.substr(-1) !== '/') {
        base = base + '/';
    }

    if (path.substr(0, 1) === '/') {
        path = path.substr(1);
    }

    return base + path;
}