import fetch from 'isomorphic-fetch';

/*
 * Actions
 */

export const DATASOURCE_REQUEST = 'DATASOURCE_REQUEST';
export const DATASOURCE_RESPONSE = 'DATASOURCE_RESPONSE';
export const DATASOURCE_ERROR = 'DATASOURCE_ERROR';
export const FETCH_ERROR = 'FETCH_ERROR';

function dataSourceRequest(path, params) {
    return {
        type: DATASOURCE_REQUEST,
        path,
        params
    };
}

function dataSourceResponse(path, response, params) {
    return {
        type: DATASOURCE_RESPONSE,
        path,
        data: response.data,
        meta: response.meta,
        params
    };
}

export function fetchError(response, id) {
    //logging, react-router pushState
    return {
        type: FETCH_ERROR,
        response,
        id
    }
}

/*
 * Action Creators
 */

export function updateDataSource(path, newParams, endpoint='') {
    return function (dispatch, getState) {
        let state = getState();

        const token = state.auth.token;

        // We'll store the new filter/sort/page state in this variable until we can be sure the data source is successfully updated.
        const params = Object.assign({}, state.dataSources[path] ? state.dataSources[path].params : {}, newParams);

        const requestOptions = {
            method: 'GET',
            mode: 'cors',
            headers: {
                'Content-type': 'application/json',
                'Accept': 'application/json'
            }
        };

        dispatch(dataSourceRequest(path, params));

        return fetch(urlWithParams(path, params, endpoint), applyToken(requestOptions, token))
            .then(response => {
                return response.json()
            })
            .then(json => {
                dispatch(dataSourceResponse(path, json, params));
            })
            .catch(response => dispatch(fetchError(response)));
    }
}

/**
 * Helper Functions
 */

export function applyHeaders(request, headers) {
    return { ...request, headers: { ...request.headers, ...headers } };
}

export function applyToken(request, token) {
    return applyHeaders(request, { 'Authorization': 'Bearer ' + token });
}


export function urlWithParams(path, params=[], endpoint='') {
    var formattedParams = [];

    for (var key in params) {
        if (params.hasOwnProperty(key)) {
            formattedParams.push(key + '=' + params[key]);
        }
    }

    // If we have some params to add to the URL, append everything with a ?, otherwise
    // just return the url
    if (formattedParams.length == 0) {
        return buildUrl(endpoint, path);
    } else {
        return buildUrl(endpoint, path) + '?' + formattedParams.join('&');
    }

}

function buildUrl(base, path) {
    if(base.substr(-1) !== '/') {
        base = base + '/';
    }

    if(path.substr(0, 1) === '/') {
        path = path.substr(1);
    }

    return base + path;
}